#include "CommonUtils/TabInterpolator.hpp"
#include "CommonUtils/FileCommentRemover.hpp"
#include "EndfUtils/Endf6Parser/Endf6Parser.hpp"

using namespace frendy;

////////////////////////////////////////////////////////////////////////////////////////////////////

void read_mt_list( string file_name, vector<Integer>& mt_list )
{
  string mod_name = file_name + "aaabbbcccdddeeefff";

  FileCommentRemover remove_obj;
  remove_obj.remove_comment(file_name, mod_name);

  ifstream fin;
  fin.open(mod_name.c_str());

  int mt_val = 0;
  while( !fin.eof() )
  {
    fin >> mt_val;
    if( mt_val > 0 && mt_val < 1000 )
    {
      mt_list.push_back(mt_val);
      mt_val = 0;
    }
  }

  fin.close();
  remove(mod_name.c_str());
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void set_xs_type_list(Endf6Parser& endf_obj,  vector<Integer>& xs_type_list)
{
  //Get MF3 data
  vector<MF03Parser> mf03 = endf_obj.get_mf03_data();

  xs_type_list.clear();
  for(int i=0; i<static_cast<int>(mf03.size()); i++)
  {
    Integer mt_no = mf03[i].get_mt_no();
    if( mt_no != unassigned_mt_no )
    {
      xs_type_list.push_back(mt_no);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void set_xs_data(Endf6Parser& endf_obj,  vector<Integer> xs_type_list,
                 vector<vector<Real> >& e_int_xs, vector<vector<Real> >& xs_tab)
{
  //Resize xs data
  int xs_data_no = static_cast<int>(xs_type_list.size());
  e_int_xs.resize(xs_data_no);
  xs_tab.resize(xs_data_no);

  //Get MF3 data
  vector<MF03Parser> mf03 = endf_obj.get_mf03_data();
  for(int i=0; i<static_cast<int>(mf03.size()); i++)
  {
    Integer mt_no = mf03[i].get_mt_no();
    for(int j=0; j<xs_data_no; j++)
    {
      if( mt_no == xs_type_list[j] )
      {
        e_int_xs[j] = mf03[i].get_E_INT_SIG();
        xs_tab[j]   = mf03[i].get_SIG_TAB();
        break;
      }
    }
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void output_result(string out_name, vector<Integer> xs_type_list, vector<vector<Real> >& ene_vec,
             vector<vector<Real> >& xs_ref_vec, vector<vector<Real> >& xs_comp_vec,
             vector<vector<Real> >& xs_dif_vec)
{
  int xs_data_no = static_cast<int>(xs_type_list.size());
  for(int i=0; i<xs_data_no; i++)
  {
    ostringstream  oss;
    oss << xs_type_list[i];

    string out_name_mod = out_name + "_mt";
    if( xs_type_list[i] < 10 )
    {
      out_name_mod = out_name_mod + "00";
    }
    else if( xs_type_list[i] < 100 )
    {
      out_name_mod = out_name_mod + "0";
    }
    out_name_mod = out_name_mod + oss.str() + ".out";

    ofstream fout;
    fout.open(out_name_mod.c_str());

    fout.precision(10);
    fout.setf(ios::showpoint);
    fout.setf(ios::scientific);

    int j_max = static_cast<int>(ene_vec[i].size());
    for(int j=0; j<j_max; j++)
    {
      fout << ene_vec[i][j] << "  "
           << xs_comp_vec[i][j] << "  " << xs_ref_vec[i][j] << "  " << xs_dif_vec[i][j] << endl;
    }
    fout.close();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

void write_xs(string pendf_name, vector<Integer> xs_type_list)
{
  cout << "  Read PENDF file : " << pendf_name << endl;
  //Allocate Endf6Parser class to endf_obj;
  Endf6Parser endf_obj;

  //Set pendf file name
  endf_obj.set_file_name_xs_only(pendf_name);

  cout << "  Copy cross section data from PENDF file" << endl;
  if( static_cast<int>(xs_type_list.size()) == 0 )
  {
    set_xs_type_list(endf_obj,  xs_type_list);
  }

  vector<vector<Real> > ene_vec, xs_vec;
  set_xs_data(endf_obj, xs_type_list, ene_vec, xs_vec);

  cout << "  Output cross section data" << endl;
  int i_max = static_cast<int>(xs_type_list.size());
  for(int i=0; i<i_max; i++)
  {
    string output = pendf_name + "_";
    if( xs_type_list[i] < 100 )
    {
      output = output + "0";
      if( xs_type_list[i] < 10 )
      {
        output = output + "0";
      }
    }

    ostringstream oss;
    oss << xs_type_list[i];
    output = output + oss.str() + ".xs";

    ofstream fout;
    fout.open(output.c_str());
    fout.setf(ios::showpoint);
    fout.setf(ios::scientific);
    int j_max = static_cast<int>(ene_vec[i].size());
    for(int j=0; j<j_max; j++)
    {
      fout << ene_vec[i][j] << "  " << xs_vec[i][j] << endl;
    }
    fout.close();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])
{
  ErrorManager err_obj;
  //err_obj.set_err_mes_opt(err_obj.err_mes_debug);
  err_obj.set_err_mes_opt(err_obj.err_mes_default);
  //err_obj.set_err_mes_opt(err_obj.err_mes_skip_warning);

  //Read PENDF file name
  if( argc <= 1 )
  {
    throw runtime_error("Please set PENDF file name.");
  }
  string pendf_name = argv[1];

  //Read MT number list
  vector<Integer> mt_list;
  mt_list.clear();
  if( argc > 2 )
  {
    string mt_list_name = argv[2];
    read_mt_list(mt_list_name, mt_list);

    if( static_cast<int>(mt_list.size()) == 0 )
    {
      cout << "Though user sets mt-list file, there is no mt data." << endl;
      cout << "This program compares in all mt data." << endl;
    }
  }

  //Read and write cross section data
  write_xs(pendf_name, mt_list);

  return 0;
}

